@@ -107,6 +107,11 @@ class DSI():
             'cbaror': 'vertical', # Colorbar orientation
             'cbarpad': 0.12,      # Colorbar padding
             'mycmap': 'viridis',  # Use your own cmap (input name of cmap as str)
+            'cmapmax': None,       # max colorbar scale
+            'cmapmin': None,       # min colorbar scale
+            'cmapext': 'neither',  # if extended cbar is desired
+            'cmapextmax': 'red',   # color of the "over max values"
+            'cmapextmin': 'green', # color of the "under min values"
             # Satisfied samples
             'satlabel': 'Sat',     # Satisfied samples label
             'satcolor': '#FF9000', # Satisfied samples color
@@ -289,7 +294,13 @@ class DSI():
             else:
                 hmvdf = pd.concat([sat, vio], axis = 0)
             hmvdf = hmvdf[opt['hmv']]
-            norm = matplotlib.colors.Normalize(vmin = hmvdf.min(), vmax = hmvdf.max())
+            cmapvmin = hmvdf.min()
+            cmapvmax = hmvdf.max()
+            if opt['cmapmax'] != None:
+                cmapvmax = opt['cmapmax']
+            if opt['cmapmin'] != None:
+                cmapvmin = opt['cmapmin']
+            norm = matplotlib.colors.Normalize(vmin = cmapvmin, vmax = cmapvmax)
             if opt['mycmap'] != None:
                 if type(opt['mycmap']) == str:
                     cmap = plt.cm.get_cmap(opt['mycmap'], 256)
@@ -298,6 +309,13 @@ class DSI():
             else:
                 cmap = self.cmap_opt[opt['cmap']]
             sm = plt.cm.ScalarMappable(cmap = cmap, norm = norm)
+            if opt['cmapext'] == 'both':
+                sm.cmap.set_over(opt['cmapextmax'])
+                sm.cmap.set_under(opt['cmapextmin'])
+            elif opt['cmapext'] == 'max':
+                sm.cmap.set_over(opt['cmapextmax'])
+            elif opt['cmapext'] == 'min':
+                sm.cmap.set_under(opt['cmapextmin'])
             
             # ----- Calculating heat map variable values in design space ----- #
             hmv_R['mean']  = sat[opt['hmv']].mean()
@@ -335,15 +353,18 @@ class DSI():
             if nosat_flag == False:
                 if (opt['hmv'] == 'None') or (opt['hidehmv'] == True):
                     ax.scatter(*zip(*sat[vnames].to_numpy()), marker = opt['satmarker'],\
-                        facecolors = opt['satfill'], label = opt['satlabel'],\
-                        color = opt['satcolor'], alpha = opt['alpha'],\
+                        facecolors = opt['satfill'], label = opt['satlabel'], color = opt['satcolor'], alpha = opt['alpha'], 
                         zorder = opt['satzorder'])
                 else:
-                    fig.colorbar(sm, label = opt['hmvlabel'],\
-                        location = opt['cbarloc'], orientation = opt['cbaror'], pad = opt['cbarpad'])
-                    ax.scatter(*zip(*sat[vnames].to_numpy()), marker = opt['satmarker'],\
-                        label = opt['satlabel'], color = cmap(norm(sat[opt['hmv']])),\
-                        alpha = opt['alpha'], zorder = opt['satzorder'])                
+                    self.cbar = fig.colorbar(sm, label = opt['hmvlabel'], location = opt['cbarloc'], orientation = opt['cbaror'], pad = opt['cbarpad'], extend = opt['cmapext'])
+                    if opt['cmapext'] == 'both':
+                        self.cbar.cmap.set_over(opt['cmapextmax'])
+                        self.cbar.cmap.set_under(opt['cmapextmin'])
+                    elif opt['cmapext'] == 'max':
+                        self.cbar.cmap.set_over(opt['cmapextmax'])
+                    elif opt['cmapext'] == 'min':
+                        self.cbar.cmap.set_under(opt['cmapextmin'])
+                    ax.scatter(*zip(*sat[vnames].to_numpy()), marker = opt['satmarker'], label = opt['satlabel'], color = cmap(norm(sat[opt['hmv']])), alpha = opt['alpha'], zorder = opt['satzorder'])                
         if opt['hidevio'] == False:
             if novio_flag == False:
                 if (opt['hmv'] == 'None') or (opt['hidehmv'] == True):
@@ -426,7 +447,9 @@ class DSI():
         """
         import numpy as np
         import pandas as pd
-        
+        from time import time
+        start_t = time()
+
         sat = self.sat
         if vnames == None:
             vnames = self.vnames
@@ -519,6 +542,10 @@ class DSI():
         self.report['space_size'] = space_size
         self.report['sol_flag'] = sol_flag
         # self.report['opt_log'] = opt_log
+        
+        end_t = time()
+        comp_t = end_t - start_t
+        self.report['time'] = comp_t
         return shp
     
     def alphashape_2D(self, P, alpha):
@@ -788,6 +815,8 @@ class DSI():
         Checks whether point x lies within the hull or not.
         """
         import numpy as np
+        from time import time
+        start_t = time()
         vn = self.vnames
         opt = self.opt
         sat = self.sat
@@ -903,8 +932,12 @@ class DSI():
                     hmv_fs_R['fs_all_samples'] = fs_df
             fs_R = {'x': x, 'FR': FR, 'rmax': rmax, 'rmin': rmin, 'space_size': fs_size, 
                 'plusmin': plusmin, 'nosam': fs_df.shape[0], 'hmv': hmv_fs_R, 
-                'hmv_sam_flag': no_samples_flag, 'not_in_region_flag': not_in_region_flag, 'AORopt_log': AORopt_log}
-
+                'hmv_sam_flag': no_samples_flag, 'not_in_region_flag': not_in_region_flag, 'AORopt_log': AORopt_log, 'fs_df': fs_df}
+        
+        end_t = time()
+        comp_t = end_t - start_t
+        fs_R.update({'time': comp_t})
+        self.inAOR = fs_df
         self.all_x.update({str(x): fs_R})
         return fs_R
         
@@ -926,6 +959,7 @@ class DSI():
             fs_R = self.all_x[str(x)]
             if fs_R['not_in_region_flag']:
                 print('x is not inside DSp.')
+        self.inAOR = fs_R['fs_df']
         FR = fs_R['FR']
         not_in_region_flag = fs_R['not_in_region_flag']
         
